#!/usr/bin/perl -w

use strict;

use lib 'lib';
use lib qw(../lib);
use Event::RPC::Client;
use Getopt::Std;

my $USAGE = <<__EOU;

Usage: client.pl [-s]

Description:
  Event::RPC client demonstration program. Execute this from
  the distribution's base or examples/ directory after starting
  the correspondent examples/server.pl program.

Options:
  -s       Use SSL encryption

__EOU

sub HELP_MESSAGE {
	my ($fh) = @_;
	$fh ||= \*STDOUT;
	print $fh $USAGE;
	exit;
}

main: {
    my %opts;
    my $opts_ok = getopts('l:s',\%opts);
   
    HELP_MESSAGE() unless $opts_ok;

    my $ssl = $opts{s} || 0;

    #-- This connects to the server, requests the exported
    #-- interfaces and establishes correspondent proxy methods
    #-- in the correspondent packages.
    my $client;
    $client = Event::RPC::Client->connect (
      server   => "localhost",
      port     => 5555,
      ssl      => $ssl,
      error_cb => sub {
      	print "An RPC error occured: $_[0]";
	print "Disconnect and exit.\n";
	$client->disconnect if $client;
	exit
      },
    );

    #-- So the call to Event::RPC::Test->new is handled transparently
    #-- by Event::RPC::Client
    my $object = Event::RPC::Test->new (
	    data => "Some test data. " x 6
    );

    #-- and methods calls as well...
    print "hello=".$object->hello,"\n";
    $object->set_data ("changed data");
    print "data=".$object->get_data."\n";

    #-- disconnection is handled by the destructor of $client	
}
