#!/usr/bin/perl -w

use strict;

use strict;
use lib qw(lib);
use lib qw(../lib);
use Event::RPC::Server;
use Event::RPC::Logger;
use Getopt::Std;

my $USAGE = <<__EOU;

Usage: server.pl [-l log-level] [-s] [-L loop-module] 

Description:
  Event::RPC server demonstration program. Execute this from
  the distribution's base or examples/ directory. Then execute
  examples/client.pl on another console.

Options:
  -l log-level       Logging level. Default: 4
  -s                 Use SSL encryption
  -L loop-module     Event loop module to use.
                     Default: Event::RPC::Loop::Event

__EOU

sub HELP_MESSAGE {
	my ($fh) = @_;
	$fh ||= \*STDOUT;
	print $fh $USAGE;
	exit;
}

main: {
    my %opts;
    my $opts_ok = getopts('L:l:s',\%opts);
   
    HELP_MESSAGE() unless $opts_ok;

    my %ssl_args;
    if ( $opts{s} ) {
      %ssl_args = (
        ssl => 1,
	ssl_key_file  => 'ssl/server.key',
	ssl_cert_file => 'ssl/server.crt',
	ssl_passwd_cb => sub { 'eventrpc' },
      );
      if ( not -f 'ssl/server.key' ) {
        chdir ("examples");
	if ( not -f 'ssl/server.key' ) {
	  print "please execute from toplevel or examples/ directory\n";
	  exit 1;
	}
      }
    }

    #-- Create a logger object
    my $logger = Event::RPC::Logger->new (
	    min_level => ($opts{l}||4),
	    fh_lref   => [ \*STDOUT ],
    );

    #-- Create a Server instance and declare the
    #-- exported interface
    my $server = Event::RPC::Server->new (
      name               => "test daemon",
      port               => 5555,
      logger             => $logger,
      loop               => $opts{L},
      start_log_listener => 1,
      %ssl_args,
      classes => {
	'Event::RPC::Test' => {
	  new       => '_constructor',
	  set_data  => 1,
	  get_data  => 1,
	  hello     => 1,
	},
      },
    );

    #-- Start the server resp. the Event loop.
    $server->start;
}

1;

